@@ -281,13 +281,13 @@ typedef struct MacroBlockD {
     int                      mi_row; // Row position in mi units
     int                      mi_col; // Column position in mi units
     uint8_t                  neighbors_ref_counts[TOTAL_REFS_PER_FRAME];
-    MbModeInfo *             above_mbmi;
-    MbModeInfo *             left_mbmi;
-    MbModeInfo *             chroma_above_mbmi;
-    MbModeInfo *             chroma_left_mbmi;
-    FRAME_CONTEXT *          tile_ctx;
-    TXFM_CONTEXT *           above_txfm_context;
-    TXFM_CONTEXT *           left_txfm_context;
+    MbModeInfo              *above_mbmi;
+    MbModeInfo              *left_mbmi;
+    MbModeInfo              *chroma_above_mbmi;
+    MbModeInfo              *chroma_left_mbmi;
+    FRAME_CONTEXT           *tile_ctx;
+    TXFM_CONTEXT            *above_txfm_context;
+    TXFM_CONTEXT            *left_txfm_context;
     struct macroblockd_plane plane[MAX_MB_PLANE];
     BlockSize                sb_type;
 } MacroBlockD;
@@ -315,13 +315,13 @@ typedef struct IntraBcContext {
     // Store the second best motion vector during full-pixel motion search
     IntMv        second_best_mv;
     MacroBlockD *xd;
-    int *        nmv_vec_cost;
-    int **       mv_cost_stack;
+    int         *nmv_vec_cost;
+    int        **mv_cost_stack;
     // buffer for hash value calculation of a block
     // used only in svt_av1_get_block_hash_value()
     // [first hash/second hash]
     // [two buffers used ping-pong]
-    uint32_t *     hash_value_buffer[2][2];
+    uint32_t      *hash_value_buffer[2][2];
     uint8_t        is_exhaustive_allowed;
     CRC_CALCULATOR crc_calculator1;
     CRC_CALCULATOR crc_calculator2;
@@ -332,14 +332,14 @@ typedef struct IntraBcContext {
 typedef struct BlkStruct {
     TransformUnit          txb_array[TRANSFORM_UNIT_MAX_COUNT]; // ec
     PredictionUnit         prediction_unit_array[MAX_NUM_OF_PU_PER_CU]; // ec
-    PaletteInfo *          palette_info; // ec
+    PaletteInfo           *palette_info; // ec
     uint8_t                palette_mem; // status of palette info alloc
     uint8_t                palette_size[2];
     IntMv                  predmv[2]; // ec
-    MacroBlockD *          av1xd;
+    MacroBlockD           *av1xd;
     InterInterCompoundData interinter_comp; // ec
     uint32_t               interp_filters; // ec
-    int32_t                interintra_wedge_index; // ec
+    uint8_t                interintra_wedge_index; // ec
     // uint8_t ref_mv_count[MODE_CTX_REF_FRAMES];
     int16_t inter_mode_ctx[MODE_CTX_REF_FRAMES]; // ec
     uint16_t
@@ -351,17 +351,17 @@ typedef struct BlkStruct {
 
     unsigned skip_flag_context : 2; // to do
     unsigned prediction_mode_flag : 2; // ec
-    unsigned block_has_coeff : 1; // ec
+    unsigned
+        block_has_coeff : 1; // ec; skip coeff only. as defined in section 6.10.11 of the av1 text
     unsigned split_flag_context : 2; // to do
 
     uint8_t qindex; // ec
     uint8_t split_flag;
-    uint8_t skip_flag; // ec
-    uint8_t mdc_split_flag; // ?
+    uint8_t skip_mode; // ec; skips mode_info + coeff. as defined in section 6.10.10 of the av1 text
     EbPictureBufferDesc
         *coeff_tmp; // buffer to store quantized coeffs from MD for the final mode of each block
     EbPictureBufferDesc
-        *   recon_tmp; // buffer to store recon from MD for the final mode of each block
+           *recon_tmp; // buffer to store recon from MD for the final mode of each block
     uint8_t drl_index; // ec
     int8_t  drl_ctx[2]; // Store the drl ctx in coding loop to avoid storing
         // final_ref_mv_stack and ref_mv_count for EC
@@ -411,7 +411,7 @@ typedef struct SuperBlock {
 
     BlkStruct *final_blk_arr;
     //for memory free only
-    MacroBlockD *  av1xd;
+    MacroBlockD   *av1xd;
     PartitionType *cu_partition_array;
     unsigned       picture_left_edge_flag : 1;
     unsigned       picture_top_edge_flag : 1;
@@ -420,16 +420,18 @@ typedef struct SuperBlock {
     unsigned       origin_x : 32;
     unsigned       origin_y : 32;
     uint8_t        qindex;
-    // Quantized Coefficients
-    TileInfo tile_info;
+    TileInfo       tile_info;
+    uint16_t       final_blk_cnt; // number of block(s) posted from EncDec to EC
 } SuperBlock;
 
-extern EbErrorType largest_coding_unit_ctor(SuperBlock *larget_coding_unit_ptr, uint8_t sb_sz,
+extern EbErrorType largest_coding_unit_ctor(SuperBlock *larget_coding_unit_ptr, uint8_t sb_size,
                                             uint16_t sb_origin_x, uint16_t sb_origin_y,
                                             uint16_t sb_index, uint8_t enc_mode,
                                             uint16_t                  max_block_cnt,
                                             struct PictureControlSet *picture_control_set);
 
+bool svt_aom_get_disallow_4x4(EncMode enc_mode, SliceType slice_type);
+bool svt_aom_get_disallow_nsq(EncMode enc_mode, bool is_islice);
 #ifdef __cplusplus
 }
 #endif
