@@ -27,8 +27,6 @@
 #include "EbCommonUtils.h"
 #include "EbResize.h"
 #include "EbInvTransforms.h"
-
-uint8_t get_disallow_4x4(EncMode enc_mode, SliceType slice_type);
 uint8_t get_bypass_encdec(EncMode enc_mode, uint8_t hbd_mode_decision, uint8_t encoder_bit_depth);
 
 #define MAX_MESH_SPEED 5 // Max speed setting for mesh motion method
@@ -421,14 +419,14 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
         ppcs->frm_hdr.use_ref_frame_mvs = 0;
     } else {
         if (fast_decode == 0) {
-            if (enc_mode <= ENC_M5)
+            if (enc_mode <= ENC_M9)
                 ppcs->frm_hdr.use_ref_frame_mvs = 1;
             else {
                 uint64_t avg_me_dist = 0;
-                for (uint16_t b64_idx = 0; b64_idx < ppcs->sb_total_count; b64_idx++) {
+                for (uint16_t b64_idx = 0; b64_idx < ppcs->b64_total_count; b64_idx++) {
                     avg_me_dist += ppcs->me_64x64_distortion[b64_idx];
                 }
-                avg_me_dist /= ppcs->sb_total_count;
+                avg_me_dist /= ppcs->b64_total_count;
                 avg_me_dist /= pcs_ptr->picture_qp;
 
                 ppcs->frm_hdr.use_ref_frame_mvs = avg_me_dist < 200 ||
@@ -439,10 +437,10 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
         } else {
             if (enc_mode <= ENC_M9) {
                 uint64_t avg_me_dist = 0;
-                for (uint16_t b64_idx = 0; b64_idx < ppcs->sb_total_count; b64_idx++) {
+                for (uint16_t b64_idx = 0; b64_idx < ppcs->b64_total_count; b64_idx++) {
                     avg_me_dist += ppcs->me_64x64_distortion[b64_idx];
                 }
-                avg_me_dist /= ppcs->sb_total_count;
+                avg_me_dist /= ppcs->b64_total_count;
                 avg_me_dist /= pcs_ptr->picture_qp;
 
                 ppcs->frm_hdr.use_ref_frame_mvs = avg_me_dist < 50 ||
@@ -484,7 +482,7 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
         pcs_ptr->pic_filter_intra_level = scs_ptr->filter_intra_level;
 
     if (fast_decode == 0 || input_resolution <= INPUT_SIZE_360p_RANGE) {
-        if (pcs_ptr->enc_mode <= ENC_M3)
+        if (pcs_ptr->enc_mode <= ENC_M4)
             pcs_ptr->parent_pcs_ptr->partition_contexts = PARTITION_CONTEXTS;
         else
             pcs_ptr->parent_pcs_ptr->partition_contexts = 4;
@@ -535,7 +533,9 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
             }
         }
     }
-
+    if (hierarchical_levels <= 2) {
+        pcs_ptr->wm_level = enc_mode <= ENC_M7 ? pcs_ptr->wm_level : 0;
+    }
     Bool enable_wm = pcs_ptr->wm_level ? 1 : 0;
     if (scs_ptr->enable_warped_motion != DEFAULT)
         enable_wm = (Bool)scs_ptr->enable_warped_motion;
@@ -559,7 +559,7 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
         if (fast_decode == 0 || input_resolution <= INPUT_SIZE_360p_RANGE) {
             if (ppcs->enc_mode <= ENC_M3)
                 ppcs->pic_obmc_level = 1;
-            else if (ppcs->enc_mode <= ENC_M4)
+            else if (ppcs->enc_mode <= ENC_M5)
                 ppcs->pic_obmc_level = 2;
             else if (enc_mode <= ENC_M6)
                 ppcs->pic_obmc_level = 3;
@@ -582,7 +582,7 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
     frm_hdr->is_motion_mode_switchable = frm_hdr->is_motion_mode_switchable || ppcs->pic_obmc_level;
 
     ppcs->bypass_cost_table_gen = 0;
-    if (enc_mode <= ENC_M10)
+    if (enc_mode <= ENC_M11)
         pcs_ptr->approx_inter_rate = 0;
     else
         pcs_ptr->approx_inter_rate = 1;
@@ -597,68 +597,60 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
     pcs_ptr->cand_reduction_level = 0;
     if (is_islice)
         pcs_ptr->cand_reduction_level = 0;
-    else if (enc_mode <= ENC_M5)
+    else if (enc_mode <= ENC_M3)
         pcs_ptr->cand_reduction_level = 0;
-    else if (enc_mode <= ENC_M8) {
-        if (pcs_ptr->coeff_lvl == LOW_LVL)
-            pcs_ptr->cand_reduction_level = 0;
-        else
-            pcs_ptr->cand_reduction_level = 1;
-    } else if (enc_mode <= ENC_M10)
+    else if (enc_mode <= ENC_M5)
         pcs_ptr->cand_reduction_level = 1;
-    else
+    else if (enc_mode <= ENC_M9) {
+        if (pcs_ptr->coeff_lvl == LOW_LVL)
+            pcs_ptr->cand_reduction_level = 1;
+        else
+            pcs_ptr->cand_reduction_level = 2;
+    } else
         pcs_ptr->cand_reduction_level = 2;
     if (scs_ptr->rc_stat_gen_pass_mode)
-        pcs_ptr->cand_reduction_level = 5;
+        pcs_ptr->cand_reduction_level = 6;
     // Set the level for the txt search
     pcs_ptr->txt_level = 0;
-    if (enc_mode <= ENC_M2)
+    if (enc_mode <= ENC_MR)
         pcs_ptr->txt_level = 1;
-    else if (enc_mode <= ENC_M3) {
+    else if (enc_mode <= ENC_M1)
+        pcs_ptr->txt_level = 2;
+    else if (enc_mode <= ENC_M2)
+        pcs_ptr->txt_level = 3;
+    else if (enc_mode <= ENC_M5) {
         if (pcs_ptr->coeff_lvl == LOW_LVL) {
-            pcs_ptr->txt_level = 1;
+            pcs_ptr->txt_level = 4;
         } else if (pcs_ptr->coeff_lvl == HIGH_LVL) {
-            pcs_ptr->txt_level = 5;
+            pcs_ptr->txt_level = 7;
         } else { // regular
-            pcs_ptr->txt_level = is_base ? 1 : 3;
+            pcs_ptr->txt_level = is_base ? 4 : 5;
         }
-    } else if (enc_mode <= ENC_M9) {
+    } else if (enc_mode <= ENC_M11) {
         if (pcs_ptr->coeff_lvl == LOW_LVL) {
-            pcs_ptr->txt_level = is_base ? 1 : 3;
+            pcs_ptr->txt_level = is_base ? 4 : 5;
         } else if (pcs_ptr->coeff_lvl == HIGH_LVL) {
-            pcs_ptr->txt_level = is_base ? 6 : 8;
+            pcs_ptr->txt_level = is_base ? 8 : 10;
         } else { // regular
-            pcs_ptr->txt_level = 5;
-        }
-    } else if (enc_mode <= ENC_M10) {
-        if (pcs_ptr->coeff_lvl == LOW_LVL) {
-            pcs_ptr->txt_level = 5;
-        } else if (pcs_ptr->coeff_lvl == HIGH_LVL) {
-            pcs_ptr->txt_level = is_base ? 6 : 8;
-            if (pcs_ptr->ref_intra_percentage < 85 && !is_base &&
-                !pcs_ptr->parent_pcs_ptr->sc_class1) {
-                pcs_ptr->txt_level = 0;
-            }
-        } else { // regular
-            pcs_ptr->txt_level = is_base ? 6 : 8;
+            pcs_ptr->txt_level = 7;
         }
     } else if (enc_mode <= ENC_M12) {
         if (pcs_ptr->coeff_lvl == LOW_LVL) {
-            pcs_ptr->txt_level = is_base ? 6 : 8;
+            pcs_ptr->txt_level = is_base ? 8 : 10;
         } else if (pcs_ptr->coeff_lvl == HIGH_LVL) {
-            pcs_ptr->txt_level = is_base ? 6 : 8;
+            pcs_ptr->txt_level = is_base ? 8 : 10;
             if (pcs_ptr->ref_intra_percentage < 85 && !pcs_ptr->parent_pcs_ptr->sc_class1) {
                 pcs_ptr->txt_level = 0;
             }
         } else { // regular
-            pcs_ptr->txt_level = is_base ? 6 : 8;
+            pcs_ptr->txt_level = is_base ? 8 : 10;
             if (pcs_ptr->ref_intra_percentage < 85 && !is_base &&
                 !pcs_ptr->parent_pcs_ptr->sc_class1) {
                 pcs_ptr->txt_level = 0;
             }
         }
     } else {
-        pcs_ptr->txt_level = is_base ? 6 : 8;
+        pcs_ptr->txt_level = is_base ? 8 : 10;
         if (pcs_ptr->ref_intra_percentage < 85 && !pcs_ptr->parent_pcs_ptr->sc_class1) {
             pcs_ptr->txt_level = 0;
         }
@@ -727,9 +719,9 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
 
     // Set the level for bipred3x3 injection
     if (scs_ptr->bipred_3x3_inject == DEFAULT) {
-        if (enc_mode <= ENC_M1)
+        if (enc_mode <= ENC_M0)
             pcs_ptr->bipred3x3_injection = 1;
-        else if (enc_mode <= ENC_M4)
+        else if (enc_mode <= ENC_M3)
             pcs_ptr->bipred3x3_injection = 2;
         else
             pcs_ptr->bipred3x3_injection = 0;
@@ -742,8 +734,10 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
         if (scs_ptr->compound_level == DEFAULT) {
             if (enc_mode <= ENC_MR)
                 pcs_ptr->inter_compound_mode = 1;
-            else if (enc_mode <= ENC_M3)
+            else if (enc_mode <= ENC_M2)
                 pcs_ptr->inter_compound_mode = 3;
+            else if (enc_mode <= ENC_M5)
+                pcs_ptr->inter_compound_mode = 4;
             else
                 pcs_ptr->inter_compound_mode = 0;
         } else {
@@ -760,14 +754,18 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
             pcs_ptr->dist_based_ref_pruning = 1;
         else if (enc_mode <= ENC_M0)
             pcs_ptr->dist_based_ref_pruning = is_base ? 1 : 2;
-        else if (enc_mode <= ENC_M1)
+        else if (enc_mode <= ENC_M2)
             pcs_ptr->dist_based_ref_pruning = is_base ? 1 : 4;
-        else if (enc_mode <= ENC_M6)
-            pcs_ptr->dist_based_ref_pruning = is_base ? 2 : 4;
-        else if (pcs_ptr->coeff_lvl == LOW_LVL) {
-            pcs_ptr->dist_based_ref_pruning = is_base ? 2 : 4;
-        } else {
-            pcs_ptr->dist_based_ref_pruning = 4;
+        else if (enc_mode <= ENC_M4)
+            pcs_ptr->dist_based_ref_pruning = is_base ? 2 : 5;
+        else if (enc_mode <= ENC_M5)
+            pcs_ptr->dist_based_ref_pruning = is_base ? 2 : 6;
+        else {
+            if (pcs_ptr->coeff_lvl == LOW_LVL) {
+                pcs_ptr->dist_based_ref_pruning = is_base ? 2 : 6;
+            } else {
+                pcs_ptr->dist_based_ref_pruning = is_base ? 3 : 6;
+            }
         }
     } else {
         pcs_ptr->dist_based_ref_pruning = 0;
@@ -778,7 +776,7 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
     if (scs_ptr->spatial_sse_full_loop_level == DEFAULT)
         if (pcs_ptr->parent_pcs_ptr->sc_class1)
             pcs_ptr->spatial_sse_full_loop_level = 1;
-        else if (enc_mode <= ENC_M9)
+        else if (enc_mode <= ENC_M10)
             pcs_ptr->spatial_sse_full_loop_level = 1;
         else
             pcs_ptr->spatial_sse_full_loop_level = 0;
@@ -791,7 +789,7 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
         pcs_ptr->parent_sq_coeff_area_based_cycles_reduction_level = 0;
     else if (enc_mode <= ENC_MR)
         pcs_ptr->parent_sq_coeff_area_based_cycles_reduction_level = is_islice ? 0 : 1;
-    else if (enc_mode <= ENC_M1)
+    else if (enc_mode <= ENC_M0)
         pcs_ptr->parent_sq_coeff_area_based_cycles_reduction_level = is_islice ? 0 : 2;
     else if (enc_mode <= ENC_M3)
         pcs_ptr->parent_sq_coeff_area_based_cycles_reduction_level = is_islice ? 0
@@ -815,24 +813,23 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
     // max_part0_to_part1_dev is used to:
     // (1) skip the H_Path if the deviation between the Parent-SQ src-to-recon distortion of (1st quadrant + 2nd quadrant) and the Parent-SQ src-to-recon distortion of (3rd quadrant + 4th quadrant) is less than TH,
     // (2) skip the V_Path if the deviation between the Parent-SQ src-to-recon distortion of (1st quadrant + 3rd quadrant) and the Parent-SQ src-to-recon distortion of (2nd quadrant + 4th quadrant) is less than TH.
-    if (enc_mode <= ENC_M2)
+    if (enc_mode <= ENC_M3)
         pcs_ptr->max_part0_to_part1_dev = 0;
-    else if (enc_mode <= ENC_M3) {
-        if (pcs_ptr->coeff_lvl == LOW_LVL)
-            pcs_ptr->max_part0_to_part1_dev = 0;
-        else if (pcs_ptr->coeff_lvl == HIGH_LVL)
-            pcs_ptr->max_part0_to_part1_dev = 10;
-        else // regular
-            pcs_ptr->max_part0_to_part1_dev = 5;
-    } else {
+    else if (enc_mode <= ENC_M4) {
         if (pcs_ptr->coeff_lvl == LOW_LVL)
             pcs_ptr->max_part0_to_part1_dev = 0;
         else if (pcs_ptr->coeff_lvl == HIGH_LVL)
             pcs_ptr->max_part0_to_part1_dev = 60;
         else // regular
             pcs_ptr->max_part0_to_part1_dev = 40;
+    } else {
+        if (pcs_ptr->coeff_lvl == LOW_LVL)
+            pcs_ptr->max_part0_to_part1_dev = 80;
+        else if (pcs_ptr->coeff_lvl == HIGH_LVL)
+            pcs_ptr->max_part0_to_part1_dev = 140;
+        else // regular
+            pcs_ptr->max_part0_to_part1_dev = 120;
     }
-
     if (enc_mode <= ENC_M2)
         pcs_ptr->skip_hv4_on_best_part = 0;
     else
@@ -862,11 +859,11 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
         pcs_ptr->txs_level = 1;
     else if (enc_mode <= ENC_MR)
         pcs_ptr->txs_level = 2;
-    else if (enc_mode <= ENC_M4)
+    else if (enc_mode <= ENC_M3)
         pcs_ptr->txs_level = is_base ? 2 : 3;
     else if (enc_mode <= ENC_M5)
         pcs_ptr->txs_level = is_base ? 2 : 4;
-    else if (enc_mode <= ENC_M6)
+    else if (enc_mode <= ENC_M7)
         pcs_ptr->txs_level = is_base ? 2 : 0;
     else if (enc_mode <= ENC_M9)
         pcs_ptr->txs_level = is_islice ? 3 : 0;
@@ -875,17 +872,15 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
             pcs_ptr->txs_level = is_islice ? 5 : 0;
         else
             pcs_ptr->txs_level = is_islice ? 3 : 0;
-    } else if (enc_mode <= ENC_M12)
+    } else
         pcs_ptr->txs_level = is_islice ? 5 : 0;
-    else
-        pcs_ptr->txs_level = 0;
 
     // Set tx_mode for the frame header
     frm_hdr->tx_mode = (pcs_ptr->txs_level) ? TX_MODE_SELECT : TX_MODE_LARGEST;
     // Set the level for nic
     pcs_ptr->nic_level = get_nic_level(enc_mode, is_base, hierarchical_levels);
     // Set the level for SQ me-search
-    if (enc_mode <= ENC_M3)
+    if (enc_mode <= ENC_M2)
         pcs_ptr->md_sq_mv_search_level = 1;
     else
         pcs_ptr->md_sq_mv_search_level = 0;
@@ -899,17 +894,15 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
     // Set the level for PME search
     if (enc_mode <= ENC_MR)
         pcs_ptr->md_pme_level = 1;
-    else if (enc_mode <= ENC_M4)
-        pcs_ptr->md_pme_level = 3;
+    else if (enc_mode <= ENC_M3)
+        pcs_ptr->md_pme_level = 2;
     else if (enc_mode <= ENC_M5) {
         if (hierarchical_levels <= 3)
             pcs_ptr->md_pme_level = 6;
         else
             pcs_ptr->md_pme_level = 3;
-    } else if (enc_mode <= ENC_M12)
+    } else
         pcs_ptr->md_pme_level = 6;
-    else
-        pcs_ptr->md_pme_level = 0;
 
     // Set the level for mds0
     pcs_ptr->mds0_level = 0;
@@ -925,7 +918,7 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
     /*
        disallow_4x4
     */
-    pcs_ptr->pic_disallow_4x4 = get_disallow_4x4(enc_mode, slice_type);
+    pcs_ptr->pic_disallow_4x4 = svt_aom_get_disallow_4x4(enc_mode, slice_type);
     /*
        Bypassing EncDec
     */
@@ -947,7 +940,7 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
     /*
         set lpd0_level
     */
-    if (enc_mode <= ENC_M4)
+    if (enc_mode <= ENC_M3)
         pcs_ptr->pic_lpd0_lvl = 0;
     else if (enc_mode <= ENC_M6)
         pcs_ptr->pic_lpd0_lvl = 1;
@@ -975,7 +968,7 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
         } else { // Regular
             pcs_ptr->pic_lpd0_lvl = (is_base || transition_present) ? 5 : 6;
         }
-    } else if (enc_mode <= ENC_M12) {
+    } else {
         if (pcs_ptr->coeff_lvl == LOW_LVL) {
             pcs_ptr->pic_lpd0_lvl = (is_base || transition_present) ? 5 : 6;
         } else if (pcs_ptr->coeff_lvl == HIGH_LVL) {
@@ -983,8 +976,8 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
         } else { // regular
             pcs_ptr->pic_lpd0_lvl = (is_base || transition_present) ? 5 : 7;
         }
-    } else
-        pcs_ptr->pic_lpd0_lvl = 7;
+    }
+
     if (pcs_ptr->parent_pcs_ptr->sc_class1 || scs_ptr->static_config.pass == ENC_MIDDLE_PASS)
         pcs_ptr->pic_skip_pd0 = 0;
     else if (enc_mode <= ENC_M13)
@@ -1014,29 +1007,25 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
                 if (enc_mode <= ENC_M2)
                     pcs_ptr->pic_depth_removal_level = 0;
                 else if (enc_mode <= ENC_M5) {
-                    if (pcs_ptr->coeff_lvl == LOW_LVL) {
-                        pcs_ptr->pic_depth_removal_level = 0;
-                    } else {
-                        if (input_resolution <= INPUT_SIZE_480p_RANGE)
-                            pcs_ptr->pic_depth_removal_level = 0;
-                        else
-                            pcs_ptr->pic_depth_removal_level = 2;
-                    }
+                    if (input_resolution <= INPUT_SIZE_480p_RANGE)
+                        pcs_ptr->pic_depth_removal_level = 1;
+                    else
+                        pcs_ptr->pic_depth_removal_level = 2;
                 } else if (enc_mode <= ENC_M7) {
                     if (pcs_ptr->coeff_lvl == LOW_LVL) {
                         if (input_resolution <= INPUT_SIZE_480p_RANGE)
-                            pcs_ptr->pic_depth_removal_level = 0;
+                            pcs_ptr->pic_depth_removal_level = 1;
                         else
                             pcs_ptr->pic_depth_removal_level = 2;
                     } else {
-                        if (input_resolution <= INPUT_SIZE_1080p_RANGE)
+                        if (input_resolution <= INPUT_SIZE_480p_RANGE)
                             pcs_ptr->pic_depth_removal_level = 1;
                         else
                             pcs_ptr->pic_depth_removal_level = 2;
                     }
                 } else if (enc_mode <= ENC_M9) {
                     if (pcs_ptr->coeff_lvl == LOW_LVL) {
-                        if (input_resolution <= INPUT_SIZE_1080p_RANGE)
+                        if (input_resolution <= INPUT_SIZE_480p_RANGE)
                             pcs_ptr->pic_depth_removal_level = 1;
                         else
                             pcs_ptr->pic_depth_removal_level = 2;
@@ -1072,7 +1061,7 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
                     pcs_ptr->pic_depth_removal_level = 0;
                 else if (enc_mode <= ENC_M5) {
                     if (input_resolution <= INPUT_SIZE_480p_RANGE)
-                        pcs_ptr->pic_depth_removal_level = 0;
+                        pcs_ptr->pic_depth_removal_level = 1;
                     else
                         pcs_ptr->pic_depth_removal_level = 2;
                 } else if (enc_mode <= ENC_M7) {
@@ -1109,37 +1098,51 @@ EbErrorType signal_derivation_mode_decision_config_kernel_oq(SequenceControlSet
             }
         }
     }
-
     if (pcs_ptr->parent_pcs_ptr->sc_class1) {
         if (enc_mode <= ENC_M6)
             pcs_ptr->pic_block_based_depth_refinement_level = 0;
         else if (enc_mode <= ENC_M9)
-            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 0 : 4;
+            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 0 : 5;
         else if (enc_mode <= ENC_M10)
-            pcs_ptr->pic_block_based_depth_refinement_level = is_islice ? 1 : 4;
+            pcs_ptr->pic_block_based_depth_refinement_level = is_islice ? 1 : 5;
         else
-            pcs_ptr->pic_block_based_depth_refinement_level = is_islice ? 6 : 11;
+            pcs_ptr->pic_block_based_depth_refinement_level = is_islice ? 7 : 12;
     } else if (enc_mode <= ENC_M2)
         pcs_ptr->pic_block_based_depth_refinement_level = 0;
-    else if (enc_mode <= ENC_M5)
+    else if (enc_mode <= ENC_M3)
         pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 0 : 2;
-    else if (enc_mode <= ENC_M7) {
-        if (pcs_ptr->coeff_lvl == LOW_LVL) {
+    else if (enc_mode <= ENC_M4) {
+        if (pcs_ptr->coeff_lvl == LOW_LVL)
             pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 0 : 2;
-
+        else
+            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 0 : 4;
+    } else if (enc_mode <= ENC_M5) {
+        if (pcs_ptr->coeff_lvl == LOW_LVL)
+            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 0 : 3;
+        else
+            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 0 : 4;
+    } else if (enc_mode <= ENC_M7) {
+        if (pcs_ptr->coeff_lvl == LOW_LVL) {
+            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 0 : 3;
         } else { // regular
-            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 1 : 2;
+            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 1 : 4;
         }
     } else {
         if (pcs_ptr->coeff_lvl == LOW_LVL) {
-            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 1 : 2;
+            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 1 : 4;
         } else { // regular
-            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 2 : 4;
+            pcs_ptr->pic_block_based_depth_refinement_level = is_base ? 2 : 5;
         }
     }
-    if (scs_ptr->max_heirachical_level == (EB_MAX_TEMPORAL_LAYERS - 1))
+
+    if (scs_ptr->max_heirachical_level == (EB_MAX_TEMPORAL_LAYERS - 1)) {
         pcs_ptr->pic_block_based_depth_refinement_level = MAX(
             0, pcs_ptr->pic_block_based_depth_refinement_level - 1);
+    }
+    if (enc_mode <= ENC_M1)
+        pcs_ptr->pic_depth_early_exit_th = 0;
+    else
+        pcs_ptr->pic_depth_early_exit_th = 90;
 
     if (pcs_ptr->parent_pcs_ptr->sc_class1) {
         if (enc_mode <= ENC_M7)
@@ -1489,8 +1492,8 @@ void *rtime_alloc_block_hash_block_is_same(size_t size) { return malloc(size); }
 // Use me_8x8_distortion and QP to predict the coeff level per frame
 static void predict_frame_coeff_lvl(struct PictureControlSet *pcs) {
     uint64_t tot_me_8x8_dis = 0;
-    for (uint32_t sb_idx = 0; sb_idx < pcs->sb_total_count; sb_idx++) {
-        tot_me_8x8_dis += pcs->parent_pcs_ptr->me_8x8_distortion[sb_idx];
+    for (uint32_t b64_idx = 0; b64_idx < pcs->b64_total_count; b64_idx++) {
+        tot_me_8x8_dis += pcs->parent_pcs_ptr->me_8x8_distortion[b64_idx];
     }
     tot_me_8x8_dis = tot_me_8x8_dis / pcs->picture_qp;
 
