@@ -54,7 +54,7 @@ static void segmentation_map_dctor(EbPtr p) {
 
 static void svt_pcs_sb_structs_dctor(EbPtr p) {
     PictureParentControlSet *obj = (PictureParentControlSet *)p;
-    EB_FREE_ARRAY(obj->sb_params_array);
+    EB_FREE_ARRAY(obj->b64_geom);
     EB_FREE_ARRAY(obj->sb_geom);
 }
 
@@ -133,10 +133,10 @@ void recon_coef_dctor(EbPtr p) {
     EB_DELETE(obj->recon_picture16bit_ptr);
     EB_DELETE(obj->recon_picture_ptr);
 
-    for (uint16_t sb_index = 0; sb_index < obj->sb_total_count_unscaled; ++sb_index) {
+    for (uint16_t sb_index = 0; sb_index < obj->b64_total_count; ++sb_index) {
         EB_DELETE(obj->quantized_coeff[sb_index]); //OMK2
     }
-    EB_DELETE_PTR_ARRAY(obj->quantized_coeff, obj->sb_total_count_unscaled);
+    EB_DELETE_PTR_ARRAY(obj->quantized_coeff, obj->b64_total_count);
 }
 void picture_control_set_dctor(EbPtr p) {
     PictureControlSet *obj      = (PictureControlSet *)p;
@@ -213,6 +213,7 @@ void picture_control_set_dctor(EbPtr p) {
     EB_FREE_ARRAY(obj->sb_skip);
     EB_FREE_ARRAY(obj->sb_64x64_mvp);
     EB_FREE_ARRAY(obj->sb_count_nz_coeffs);
+    EB_FREE_ARRAY(obj->b64_me_qindex);
     EB_DELETE(obj->bitstream_ptr);
     EB_DELETE_PTR_ARRAY(obj->entropy_coding_info, tile_cnt);
 
@@ -308,11 +309,11 @@ EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data_ptr) {
 
     // SBs
     const uint16_t picture_sb_width  = (uint16_t)((init_data_ptr->picture_width +
-                                                  init_data_ptr->sb_sz - 1) /
-                                                 init_data_ptr->sb_sz);
+                                                  init_data_ptr->b64_size - 1) /
+                                                 init_data_ptr->b64_size);
     const uint16_t picture_sb_height = (uint16_t)((init_data_ptr->picture_height +
-                                                   init_data_ptr->sb_sz - 1) /
-                                                  init_data_ptr->sb_sz);
+                                                   init_data_ptr->b64_size - 1) /
+                                                  init_data_ptr->b64_size);
     uint16_t       sb_index;
     Bool           is_16bit = init_data_ptr->bit_depth > 8 ? TRUE : FALSE;
 
@@ -327,9 +328,9 @@ EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data_ptr) {
     input_pic_buf_desc_init_data.bit_depth          = init_data_ptr->bit_depth;
     input_pic_buf_desc_init_data.buffer_enable_mask = PICTURE_BUFFER_DESC_FULL_MASK;
     input_pic_buf_desc_init_data.color_format       = init_data_ptr->color_format;
-    uint16_t padding                                = init_data_ptr->sb_size_pix + 32;
+    uint16_t padding                                = init_data_ptr->sb_size + 32;
     if (init_data_ptr->is_scale) {
-        padding += init_data_ptr->sb_size_pix;
+        padding += init_data_ptr->sb_size;
     }
     input_pic_buf_desc_init_data.left_padding  = padding;
     input_pic_buf_desc_init_data.right_padding = padding;
@@ -364,15 +365,15 @@ EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data_ptr) {
 
     // SB Array
     // object_ptr->sb_total_count          = picture_sb_width * picture_sb_height;
-    object_ptr->sb_total_count_unscaled = picture_sb_width * picture_sb_height;
-    EB_ALLOC_PTR_ARRAY(object_ptr->quantized_coeff, object_ptr->sb_total_count_unscaled);
+    object_ptr->b64_total_count = picture_sb_width * picture_sb_height;
+    EB_ALLOC_PTR_ARRAY(object_ptr->quantized_coeff, object_ptr->b64_total_count);
 
     //object_ptr->sb_total_count_pix = all_sb;
 
     EbPictureBufferDescInitData coeff_init_data;
     coeff_init_data.buffer_enable_mask = PICTURE_BUFFER_DESC_FULL_MASK;
-    coeff_init_data.max_width          = init_data_ptr->sb_size_pix;
-    coeff_init_data.max_height         = init_data_ptr->sb_size_pix;
+    coeff_init_data.max_width          = init_data_ptr->sb_size;
+    coeff_init_data.max_height         = init_data_ptr->sb_size;
     coeff_init_data.bit_depth          = EB_THIRTYTWO_BIT;
     coeff_init_data.color_format       = init_data_ptr->color_format;
     coeff_init_data.left_padding       = 0;
@@ -380,7 +381,7 @@ EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data_ptr) {
     coeff_init_data.top_padding        = 0;
     coeff_init_data.bot_padding        = 0;
     coeff_init_data.split_mode         = FALSE;
-    for (sb_index = 0; sb_index < object_ptr->sb_total_count_unscaled; ++sb_index) {
+    for (sb_index = 0; sb_index < object_ptr->b64_total_count; ++sb_index) {
         EB_NEW(object_ptr->quantized_coeff[sb_index], //OMK2
                svt_picture_buffer_desc_ctor,
                (EbPtr)&coeff_init_data);
@@ -388,10 +389,6 @@ EbErrorType recon_coef_ctor(EncDecSet *object_ptr, EbPtr object_init_data_ptr) {
 
     return EB_ErrorNone;
 }
-
-uint8_t get_disallow_4x4(EncMode enc_mode, SliceType slice_type);
-uint8_t get_disallow_nsq(EncMode enc_mode);
-
 uint32_t get_out_buffer_size(uint32_t picture_width, uint32_t picture_height) {
     uint32_t frame_size = picture_width * picture_height * 3 / 2; //assuming 4:2:0;
     if (frame_size > INPUT_SIZE_4K_TH)
@@ -406,14 +403,14 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
     EbPictureBufferDescInitData coeff_buffer_desc_init_data;
 
     // Max/Min CU Sizes
-    const uint32_t max_blk_size = init_data_ptr->sb_size_pix;
+    const uint32_t max_blk_size = init_data_ptr->sb_size;
     // SBs
     const uint16_t picture_sb_width  = (uint16_t)((init_data_ptr->picture_width +
-                                                  init_data_ptr->sb_sz - 1) /
-                                                 init_data_ptr->sb_sz);
+                                                  init_data_ptr->b64_size - 1) /
+                                                 init_data_ptr->b64_size);
     const uint16_t picture_sb_height = (uint16_t)((init_data_ptr->picture_height +
-                                                   init_data_ptr->sb_sz - 1) /
-                                                  init_data_ptr->sb_sz);
+                                                   init_data_ptr->b64_size - 1) /
+                                                  init_data_ptr->b64_size);
     uint16_t       sb_index;
     uint16_t       sb_origin_x;
     uint16_t       sb_origin_y;
@@ -436,7 +433,7 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
     object_ptr->hash_table.p_lookup_table = NULL;
 
     // Init Picture Init data
-    uint16_t padding = init_data_ptr->sb_size_pix + 32;
+    uint16_t padding = init_data_ptr->sb_size + 32;
 
     coeff_buffer_desc_init_data.max_width          = init_data_ptr->picture_width;
     coeff_buffer_desc_init_data.max_height         = init_data_ptr->picture_height;
@@ -496,32 +493,33 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
     object_ptr->temporal_layer_index = 0;
 
     // SB Array
-    object_ptr->sb_total_count          = picture_sb_width * picture_sb_height;
-    object_ptr->sb_total_count_unscaled = object_ptr->sb_total_count;
-    EB_ALLOC_PTR_ARRAY(object_ptr->sb_ptr_array, object_ptr->sb_total_count);
-    EB_MALLOC_ARRAY(object_ptr->sb_intra, object_ptr->sb_total_count);
-    EB_MALLOC_ARRAY(object_ptr->sb_skip, object_ptr->sb_total_count);
-    EB_MALLOC_ARRAY(object_ptr->sb_64x64_mvp, object_ptr->sb_total_count);
+    object_ptr->b64_total_count = picture_sb_width * picture_sb_height;
+    EB_MALLOC_ARRAY(object_ptr->sb_intra, object_ptr->b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->sb_skip, object_ptr->b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->sb_64x64_mvp, object_ptr->b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->b64_me_qindex, object_ptr->b64_total_count);
 
     sb_origin_x = 0;
     sb_origin_y = 0;
 
     const uint16_t picture_sb_w = (uint16_t)((init_data_ptr->picture_width +
-                                              init_data_ptr->sb_size_pix - 1) /
-                                             init_data_ptr->sb_size_pix);
+                                              init_data_ptr->sb_size - 1) /
+                                             init_data_ptr->sb_size);
     const uint16_t picture_sb_h = (uint16_t)((init_data_ptr->picture_height +
-                                              init_data_ptr->sb_size_pix - 1) /
-                                             init_data_ptr->sb_size_pix);
+                                              init_data_ptr->sb_size - 1) /
+                                             init_data_ptr->sb_size);
     const uint16_t all_sb       = picture_sb_w * picture_sb_h;
 
-    object_ptr->sb_total_count_pix = all_sb;
+    object_ptr->sb_total_count          = all_sb;
+    object_ptr->sb_total_count_unscaled = all_sb;
+    EB_ALLOC_PTR_ARRAY(object_ptr->sb_ptr_array, object_ptr->sb_total_count_unscaled);
 
-    EB_MALLOC_ARRAY(object_ptr->sb_count_nz_coeffs, object_ptr->sb_total_count_pix);
+    EB_MALLOC_ARRAY(object_ptr->sb_count_nz_coeffs, object_ptr->sb_total_count);
 
     for (sb_index = 0; sb_index < all_sb; ++sb_index) {
         EB_NEW(object_ptr->sb_ptr_array[sb_index],
                largest_coding_unit_ctor,
-               (uint8_t)init_data_ptr->sb_size_pix,
+               (uint8_t)init_data_ptr->sb_size,
                (uint16_t)(sb_origin_x * max_blk_size),
                (uint16_t)(sb_origin_y * max_blk_size),
                (uint16_t)sb_index,
@@ -1178,13 +1176,17 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
 
     //the granularity is 4x4
     EB_MALLOC_ARRAY(object_ptr->mi_grid_base,
-                    all_sb * (init_data_ptr->sb_size_pix >> MI_SIZE_LOG2) *
-                        (init_data_ptr->sb_size_pix >> MI_SIZE_LOG2));
+                    all_sb * (init_data_ptr->sb_size >> MI_SIZE_LOG2) *
+                        (init_data_ptr->sb_size >> MI_SIZE_LOG2));
 
     // If NSQ is allowed, then need a 4x4 MI grid because 8x8 NSQ shapes will require 4x4 granularity
-    uint8_t disallow_4x4 = get_disallow_nsq(init_data_ptr->enc_mode);
+    bool disallow_4x4 = true;
+    for (uint8_t is_islice = 0; is_islice <= 1; is_islice++)
+        disallow_4x4 = MIN(disallow_4x4,
+                           svt_aom_get_disallow_nsq(init_data_ptr->enc_mode, is_islice));
     for (SliceType slice_type = 0; slice_type < IDR_SLICE + 1; slice_type++)
-        disallow_4x4 = MIN(disallow_4x4, get_disallow_4x4(init_data_ptr->enc_mode, slice_type));
+        disallow_4x4 = MIN(disallow_4x4,
+                           svt_aom_get_disallow_4x4(init_data_ptr->enc_mode, slice_type));
 
     object_ptr->disallow_4x4_all_frames = disallow_4x4;
 
@@ -1192,19 +1194,18 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
     8x8 blocks.  The mi_grid will still be 4x4 so that the data can be accessed the same way throughout
     the code. */
     EB_MALLOC_ARRAY(object_ptr->mip,
-                    all_sb * (init_data_ptr->sb_size_pix >> (MI_SIZE_LOG2 + disallow_4x4)) *
-                        (init_data_ptr->sb_size_pix >> (MI_SIZE_LOG2 + disallow_4x4)));
+                    all_sb * (init_data_ptr->sb_size >> (MI_SIZE_LOG2 + disallow_4x4)) *
+                        (init_data_ptr->sb_size >> (MI_SIZE_LOG2 + disallow_4x4)));
 
     memset(object_ptr->mip,
            0,
-           sizeof(ModeInfo) * all_sb *
-               (init_data_ptr->sb_size_pix >> (MI_SIZE_LOG2 + disallow_4x4)) *
-               (init_data_ptr->sb_size_pix >> (MI_SIZE_LOG2 + disallow_4x4)));
+           sizeof(ModeInfo) * all_sb * (init_data_ptr->sb_size >> (MI_SIZE_LOG2 + disallow_4x4)) *
+               (init_data_ptr->sb_size >> (MI_SIZE_LOG2 + disallow_4x4)));
 
-    uint32_t mi_stride = picture_sb_w * (init_data_ptr->sb_size_pix >> MI_SIZE_LOG2);
-    for (uint32_t mi_h = 0; mi_h < picture_sb_h * (init_data_ptr->sb_size_pix >> MI_SIZE_LOG2);
+    uint32_t mi_stride = picture_sb_w * (init_data_ptr->sb_size >> MI_SIZE_LOG2);
+    for (uint32_t mi_h = 0; mi_h < picture_sb_h * (init_data_ptr->sb_size >> MI_SIZE_LOG2);
          mi_h++) {
-        for (uint32_t mi_w = 0; mi_w < picture_sb_w * (init_data_ptr->sb_size_pix >> MI_SIZE_LOG2);
+        for (uint32_t mi_w = 0; mi_w < picture_sb_w * (init_data_ptr->sb_size >> MI_SIZE_LOG2);
              mi_w++) {
             uint32_t mi_grid_idx = mi_h * mi_stride + mi_w;
             uint32_t mip_idx     = (mi_h >> disallow_4x4) * (mi_stride >> disallow_4x4) +
@@ -1212,7 +1213,7 @@ EbErrorType picture_control_set_ctor(PictureControlSet *object_ptr, EbPtr object
             object_ptr->mi_grid_base[mi_grid_idx] = object_ptr->mip + mip_idx;
         }
     }
-    object_ptr->mi_stride = picture_sb_w * (init_data_ptr->sb_size_pix >> MI_SIZE_LOG2);
+    object_ptr->mi_stride = picture_sb_w * (init_data_ptr->sb_size >> MI_SIZE_LOG2);
     if (init_data_ptr->mfmv) {
         //MFMV: map is 8x8 based.
         uint32_t  mi_rows  = init_data_ptr->picture_height >> MI_SIZE_LOG2;
@@ -1324,11 +1325,11 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
     PictureControlSetInitData *init_data_ptr    = (PictureControlSetInitData *)object_init_data_ptr;
     EbErrorType                return_error     = EB_ErrorNone;
     const uint16_t             picture_sb_width = (uint16_t)((init_data_ptr->picture_width +
-                                                  init_data_ptr->sb_sz - 1) /
-                                                 init_data_ptr->sb_sz);
+                                                  init_data_ptr->b64_size - 1) /
+                                                 init_data_ptr->b64_size);
     const uint16_t             picture_sb_height = (uint16_t)((init_data_ptr->picture_height +
-                                                   init_data_ptr->sb_sz - 1) /
-                                                  init_data_ptr->sb_sz);
+                                                   init_data_ptr->b64_size - 1) /
+                                                  init_data_ptr->b64_size);
     const uint16_t subsampling_x = (init_data_ptr->color_format == EB_YUV444 ? 1 : 2) - 1;
     const uint16_t subsampling_y = (init_data_ptr->color_format >= EB_YUV422 ? 1 : 2) - 1;
 
@@ -1361,15 +1362,14 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
     } else
         return EB_ErrorBadParameter;
     // GOP
-    object_ptr->pred_struct_index       = 0;
-    object_ptr->picture_number          = 0;
-    object_ptr->idr_flag                = FALSE;
-    object_ptr->temporal_layer_index    = 0;
-    object_ptr->total_num_bits          = 0;
-    object_ptr->last_idr_picture        = 0;
-    object_ptr->sb_total_count          = picture_sb_width * picture_sb_height;
-    object_ptr->sb_total_count_unscaled = object_ptr->sb_total_count;
-    object_ptr->is_pcs_sb_params        = FALSE;
+    object_ptr->pred_struct_index    = 0;
+    object_ptr->picture_number       = 0;
+    object_ptr->idr_flag             = FALSE;
+    object_ptr->temporal_layer_index = 0;
+    object_ptr->total_num_bits       = 0;
+    object_ptr->last_idr_picture     = 0;
+    object_ptr->b64_total_count      = picture_sb_width * picture_sb_height;
+    object_ptr->is_pcs_sb_params     = FALSE;
 
     object_ptr->data_ll_head_ptr         = (EbLinkedListNode *)NULL;
     object_ptr->app_out_data_ll_head_ptr = (EbLinkedListNode *)NULL;
@@ -1380,7 +1380,7 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
             block_count = 85;
         else
             block_count = 1;
-        EB_MALLOC_2D(object_ptr->variance, object_ptr->sb_total_count, block_count);
+        EB_MALLOC_2D(object_ptr->variance, object_ptr->b64_total_count, block_count);
     }
 
     if (init_data_ptr->scene_change_detection) {
@@ -1412,15 +1412,15 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
     }
     object_ptr->r0 = 0;
 
-    EB_MALLOC_ARRAY(object_ptr->rc_me_distortion, object_ptr->sb_total_count);
-    EB_MALLOC_ARRAY(object_ptr->stationary_block_present_sb, object_ptr->sb_total_count);
-    EB_MALLOC_ARRAY(object_ptr->rc_me_allow_gm, object_ptr->sb_total_count);
-    EB_MALLOC_ARRAY(object_ptr->me_64x64_distortion, object_ptr->sb_total_count);
-    EB_MALLOC_ARRAY(object_ptr->me_32x32_distortion, object_ptr->sb_total_count);
-    EB_MALLOC_ARRAY(object_ptr->me_16x16_distortion, object_ptr->sb_total_count);
-    EB_MALLOC_ARRAY(object_ptr->me_8x8_distortion, object_ptr->sb_total_count);
+    EB_MALLOC_ARRAY(object_ptr->rc_me_distortion, object_ptr->b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->stationary_block_present_sb, object_ptr->b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->rc_me_allow_gm, object_ptr->b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->me_64x64_distortion, object_ptr->b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->me_32x32_distortion, object_ptr->b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->me_16x16_distortion, object_ptr->b64_total_count);
+    EB_MALLOC_ARRAY(object_ptr->me_8x8_distortion, object_ptr->b64_total_count);
 
-    EB_MALLOC_ARRAY(object_ptr->me_8x8_cost_variance, object_ptr->sb_total_count);
+    EB_MALLOC_ARRAY(object_ptr->me_8x8_cost_variance, object_ptr->b64_total_count);
     // SB noise variance array
     EB_CREATE_MUTEX(object_ptr->me_processed_b64_mutex);
     EB_CREATE_SEMAPHORE(object_ptr->temp_filt_done_semaphore, 0, 1);
@@ -1479,7 +1479,7 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
     //Jing: need to know the tile split info at pcs initialize stage
     object_ptr->log2_tile_rows = init_data_ptr->log2_tile_rows;
     object_ptr->log2_tile_cols = init_data_ptr->log2_tile_cols;
-    object_ptr->log2_sb_sz     = init_data_ptr->log2_sb_sz;
+    object_ptr->log2_sb_size   = init_data_ptr->log2_sb_size;
     set_tile_info(object_ptr);
     EB_MALLOC_ARRAY(
         object_ptr->tile_group_info,
@@ -1522,7 +1522,7 @@ EbErrorType picture_parent_control_set_ctor(PictureParentControlSet *object_ptr,
 static void me_dctor(EbPtr p) {
     MotionEstimationData *obj = (MotionEstimationData *)p;
 
-    EB_DELETE_PTR_ARRAY(obj->me_results, obj->sb_total_count_unscaled);
+    EB_DELETE_PTR_ARRAY(obj->me_results, obj->b64_total_count);
     if (obj->ois_mb_results)
         EB_FREE_2D(obj->ois_mb_results);
     if (obj->tpl_stats)
@@ -1540,16 +1540,16 @@ EbErrorType me_ctor(MotionEstimationData *object_ptr, EbPtr object_init_data_ptr
     PictureControlSetInitData *init_data_ptr    = (PictureControlSetInitData *)object_init_data_ptr;
     EbErrorType                return_error     = EB_ErrorNone;
     const uint16_t             picture_sb_width = (uint16_t)((init_data_ptr->picture_width +
-                                                  init_data_ptr->sb_sz - 1) /
-                                                 init_data_ptr->sb_sz);
+                                                  init_data_ptr->b64_size - 1) /
+                                                 init_data_ptr->b64_size);
     const uint16_t             picture_sb_height = (uint16_t)((init_data_ptr->picture_height +
-                                                   init_data_ptr->sb_sz - 1) /
-                                                  init_data_ptr->sb_sz);
+                                                   init_data_ptr->b64_size - 1) /
+                                                  init_data_ptr->b64_size);
 
     uint16_t sb_index;
-    object_ptr->dctor                   = me_dctor;
-    uint32_t sb_total_count             = picture_sb_width * picture_sb_height;
-    object_ptr->sb_total_count_unscaled = sb_total_count;
+    object_ptr->dctor           = me_dctor;
+    uint32_t sb_total_count     = picture_sb_width * picture_sb_height;
+    object_ptr->b64_total_count = sb_total_count;
 
     EB_ALLOC_PTR_ARRAY(object_ptr->me_results, sb_total_count);
 
@@ -1601,64 +1601,53 @@ EbErrorType me_ctor(MotionEstimationData *object_ptr, EbPtr object_init_data_ptr
     }
     return return_error;
 }
-EbErrorType sb_params_init_pcs(SequenceControlSet *scs_ptr, PictureParentControlSet *pcs_ptr) {
+
+EbErrorType b64_geom_init_pcs(SequenceControlSet *scs, PictureParentControlSet *pcs) {
     EbErrorType return_error = EB_ErrorNone;
-    uint16_t    sb_index;
+    uint16_t    b64_idx;
     uint16_t    raster_scan_blk_index;
-    uint16_t    encoding_width  = pcs_ptr->aligned_width;
-    uint16_t    encoding_height = pcs_ptr->aligned_height;
+    uint16_t    encoding_width  = pcs->aligned_width;
+    uint16_t    encoding_height = pcs->aligned_height;
+    uint8_t     b64_size        = scs->b64_size;
 
-    uint8_t picture_sb_width  = (uint8_t)((encoding_width + scs_ptr->sb_sz - 1) / scs_ptr->sb_sz);
-    uint8_t picture_sb_height = (uint8_t)((encoding_height + scs_ptr->sb_sz - 1) / scs_ptr->sb_sz);
+    uint8_t picture_b64_width  = (uint8_t)((encoding_width + b64_size - 1) / b64_size);
+    uint8_t picture_b64_height = (uint8_t)((encoding_height + b64_size - 1) / b64_size);
 
-    EB_FREE_ARRAY(pcs_ptr->sb_params_array);
-    EB_MALLOC_ARRAY(pcs_ptr->sb_params_array, picture_sb_width * picture_sb_height);
+    EB_FREE_ARRAY(pcs->b64_geom);
+    EB_MALLOC_ARRAY(pcs->b64_geom, picture_b64_width * picture_b64_height);
 
-    for (sb_index = 0; sb_index < picture_sb_width * picture_sb_height; ++sb_index) {
-        pcs_ptr->sb_params_array[sb_index].horizontal_index = (uint8_t)(sb_index %
-                                                                        picture_sb_width);
-        pcs_ptr->sb_params_array[sb_index].vertical_index = (uint8_t)(sb_index / picture_sb_width);
-        pcs_ptr->sb_params_array[sb_index].origin_x =
-            pcs_ptr->sb_params_array[sb_index].horizontal_index * scs_ptr->sb_sz;
-        pcs_ptr->sb_params_array[sb_index].origin_y =
-            pcs_ptr->sb_params_array[sb_index].vertical_index * scs_ptr->sb_sz;
+    for (b64_idx = 0; b64_idx < picture_b64_width * picture_b64_height; ++b64_idx) {
+        B64Geom *b64_geom          = &pcs->b64_geom[b64_idx];
+        b64_geom->horizontal_index = (uint8_t)(b64_idx % picture_b64_width);
+        b64_geom->vertical_index   = (uint8_t)(b64_idx / picture_b64_width);
+        b64_geom->origin_x         = b64_geom->horizontal_index * b64_size;
+        b64_geom->origin_y         = b64_geom->vertical_index * b64_size;
 
-        pcs_ptr->sb_params_array[sb_index].width =
-            (uint8_t)(((encoding_width - pcs_ptr->sb_params_array[sb_index].origin_x) <
-                       scs_ptr->sb_sz)
-                          ? encoding_width - pcs_ptr->sb_params_array[sb_index].origin_x
-                          : scs_ptr->sb_sz);
+        b64_geom->width = (uint8_t)(((encoding_width - b64_geom->origin_x) < b64_size)
+                                        ? encoding_width - b64_geom->origin_x
+                                        : b64_size);
 
-        pcs_ptr->sb_params_array[sb_index].height =
-            (uint8_t)(((encoding_height - pcs_ptr->sb_params_array[sb_index].origin_y) <
-                       scs_ptr->sb_sz)
-                          ? encoding_height - pcs_ptr->sb_params_array[sb_index].origin_y
-                          : scs_ptr->sb_sz);
+        b64_geom->height = (uint8_t)(((encoding_height - b64_geom->origin_y) < b64_size)
+                                         ? encoding_height - b64_geom->origin_y
+                                         : b64_size);
 
-        pcs_ptr->sb_params_array[sb_index].is_complete_sb =
-            (uint8_t)(((pcs_ptr->sb_params_array[sb_index].width == scs_ptr->sb_sz) &&
-                       (pcs_ptr->sb_params_array[sb_index].height == scs_ptr->sb_sz))
-                          ? 1
-                          : 0);
+        b64_geom->is_complete_b64 =
+            (uint8_t)(((b64_geom->width == b64_size) && (b64_geom->height == b64_size)) ? 1 : 0);
 
-        pcs_ptr->sb_params_array[sb_index].is_edge_sb =
-            (pcs_ptr->sb_params_array[sb_index].origin_x < scs_ptr->sb_sz) ||
-                (pcs_ptr->sb_params_array[sb_index].origin_y < scs_ptr->sb_sz) ||
-                (pcs_ptr->sb_params_array[sb_index].origin_x > encoding_width - scs_ptr->sb_sz) ||
-                (pcs_ptr->sb_params_array[sb_index].origin_y > encoding_height - scs_ptr->sb_sz)
+        b64_geom->is_edge_sb = (b64_geom->origin_x < b64_size) || (b64_geom->origin_y < b64_size) ||
+                (b64_geom->origin_x > encoding_width - b64_size) ||
+                (b64_geom->origin_y > encoding_height - b64_size)
             ? 1
             : 0;
 
         for (raster_scan_blk_index = RASTER_SCAN_CU_INDEX_64x64;
              raster_scan_blk_index <= RASTER_SCAN_CU_INDEX_8x8_63;
              raster_scan_blk_index++) {
-            pcs_ptr->sb_params_array[sb_index].raster_scan_blk_validity[raster_scan_blk_index] =
-                ((pcs_ptr->sb_params_array[sb_index].origin_x +
-                      raster_scan_blk_x[raster_scan_blk_index] +
+            b64_geom->raster_scan_blk_validity[raster_scan_blk_index] =
+                ((b64_geom->origin_x + raster_scan_blk_x[raster_scan_blk_index] +
                       raster_scan_blk_size[raster_scan_blk_index] >
                   encoding_width) ||
-                 (pcs_ptr->sb_params_array[sb_index].origin_y +
-                      raster_scan_blk_y[raster_scan_blk_index] +
+                 (b64_geom->origin_y + raster_scan_blk_y[raster_scan_blk_index] +
                       raster_scan_blk_size[raster_scan_blk_index] >
                   encoding_height))
                 ? FALSE
@@ -1666,10 +1655,10 @@ EbErrorType sb_params_init_pcs(SequenceControlSet *scs_ptr, PictureParentControl
         }
 
         // super-res can not work with multi-tiles, just set up it for no tiling
-        pcs_ptr->sb_params_array[sb_index].tile_start_x = 0;
-        pcs_ptr->sb_params_array[sb_index].tile_start_y = 0;
-        pcs_ptr->sb_params_array[sb_index].tile_end_x   = encoding_width;
-        pcs_ptr->sb_params_array[sb_index].tile_end_y   = encoding_height;
+        b64_geom->tile_start_x = 0;
+        b64_geom->tile_start_y = 0;
+        b64_geom->tile_end_x   = encoding_width;
+        b64_geom->tile_end_y   = encoding_height;
     }
 
     return return_error;
@@ -1682,9 +1671,8 @@ EbErrorType sb_geom_init_pcs(SequenceControlSet *scs_ptr, PictureParentControlSe
     uint16_t encoding_width  = pcs_ptr->aligned_width;
     uint16_t encoding_height = pcs_ptr->aligned_height;
 
-    uint16_t picture_sb_width  = (encoding_width + scs_ptr->sb_size_pix - 1) / scs_ptr->sb_size_pix;
-    uint16_t picture_sb_height = (encoding_height + scs_ptr->sb_size_pix - 1) /
-        scs_ptr->sb_size_pix;
+    uint16_t picture_sb_width  = (encoding_width + scs_ptr->sb_size - 1) / scs_ptr->sb_size;
+    uint16_t picture_sb_height = (encoding_height + scs_ptr->sb_size - 1) / scs_ptr->sb_size;
 
     EB_FREE_ARRAY(pcs_ptr->sb_geom);
     EB_MALLOC_ARRAY(pcs_ptr->sb_geom, picture_sb_width * picture_sb_height);
@@ -1693,25 +1681,23 @@ EbErrorType sb_geom_init_pcs(SequenceControlSet *scs_ptr, PictureParentControlSe
         pcs_ptr->sb_geom[sb_index].horizontal_index = sb_index % picture_sb_width;
         pcs_ptr->sb_geom[sb_index].vertical_index   = sb_index / picture_sb_width;
         pcs_ptr->sb_geom[sb_index].origin_x         = pcs_ptr->sb_geom[sb_index].horizontal_index *
-            scs_ptr->sb_size_pix;
+            scs_ptr->sb_size;
         pcs_ptr->sb_geom[sb_index].origin_y = pcs_ptr->sb_geom[sb_index].vertical_index *
-            scs_ptr->sb_size_pix;
+            scs_ptr->sb_size;
 
         pcs_ptr->sb_geom[sb_index].width =
-            (uint8_t)(((encoding_width - pcs_ptr->sb_geom[sb_index].origin_x) <
-                       scs_ptr->sb_size_pix)
+            (uint8_t)(((encoding_width - pcs_ptr->sb_geom[sb_index].origin_x) < scs_ptr->sb_size)
                           ? encoding_width - pcs_ptr->sb_geom[sb_index].origin_x
-                          : scs_ptr->sb_size_pix);
+                          : scs_ptr->sb_size);
 
         pcs_ptr->sb_geom[sb_index].height =
-            (uint8_t)(((encoding_height - pcs_ptr->sb_geom[sb_index].origin_y) <
-                       scs_ptr->sb_size_pix)
+            (uint8_t)(((encoding_height - pcs_ptr->sb_geom[sb_index].origin_y) < scs_ptr->sb_size)
                           ? encoding_height - pcs_ptr->sb_geom[sb_index].origin_y
-                          : scs_ptr->sb_size_pix);
+                          : scs_ptr->sb_size);
 
         pcs_ptr->sb_geom[sb_index].is_complete_sb =
-            (uint8_t)(((pcs_ptr->sb_geom[sb_index].width == scs_ptr->sb_size_pix) &&
-                       (pcs_ptr->sb_geom[sb_index].height == scs_ptr->sb_size_pix))
+            (uint8_t)(((pcs_ptr->sb_geom[sb_index].width == scs_ptr->sb_size) &&
+                       (pcs_ptr->sb_geom[sb_index].height == scs_ptr->sb_size))
                           ? 1
                           : 0);
 
