@@ -277,6 +277,12 @@ static void update_firstpass_stats(PictureParentControlSet *pcs_ptr, const FRAME
         fps.coded_error    = (double)(stats->coded_error >> 8) + min_err;
         fps.sr_coded_error = (double)(stats->sr_coded_error >> 8) + min_err;
         fps.intra_error     = (double)(stats->intra_error >> 8) + min_err;
+        // if blocks are skipped, the errors need to be updated
+        if (bypass_blk_step == 2) {
+            fps.coded_error *= 3;
+            fps.sr_coded_error *= 3;
+            fps.intra_error *= 3;
+        }
         fps.count           = 1.0;
         fps.pcnt_inter      = (double)stats->inter_count / num_mbs;
         fps.pcnt_second_ref = (double)stats->second_ref_count / num_mbs;
@@ -622,7 +628,6 @@ EbErrorType first_pass_signal_derivation_mode_decision_config_kernel(PictureCont
  ************************************************/
 void *set_first_pass_me_hme_params_oq(MeContext *me_context_ptr, SequenceControlSet *scs_ptr,
                                       EbInputResolution input_resolution) {
-    me_context_ptr->stat_factor = 100;
     // HME/ME default settings
     me_context_ptr->num_hme_sa_w = 2;
     me_context_ptr->num_hme_sa_h = 2;
@@ -871,7 +876,7 @@ static int open_loop_firstpass_inter_prediction(
         uint32_t           me_mb_offset = 0;
         BlockGeom          blk_geom;
         const MeSbResults *me_results = ppcs_ptr->pa_me_data->me_results[me_sb_addr];
-        uint32_t           me_sb_size = ppcs_ptr->scs_ptr->sb_sz;
+        uint32_t           me_sb_size = ppcs_ptr->scs_ptr->b64_size;
         blk_geom.origin_x             = blk_origin_x - (blk_origin_x / me_sb_size) * me_sb_size;
         blk_geom.origin_y             = blk_origin_y - (blk_origin_y / me_sb_size) * me_sb_size;
         blk_geom.bwidth               = FORCED_BLK_SIZE;
@@ -1029,7 +1034,7 @@ static EbErrorType first_pass_frame_seg(PictureParentControlSet *ppcs_ptr, int32
     const uint32_t blk_rows = (uint32_t)(input_picture_ptr->height + FORCED_BLK_SIZE - 1) /
         FORCED_BLK_SIZE;
 
-    uint32_t me_sb_size         = ppcs_ptr->scs_ptr->sb_sz;
+    uint32_t me_sb_size         = ppcs_ptr->scs_ptr->b64_size;
     uint32_t me_pic_width_in_sb = (ppcs_ptr->aligned_width + me_sb_size - 1) / me_sb_size;
     uint32_t me_sb_x, me_sb_y, me_sb_addr;
 
