@@ -122,6 +122,12 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs_ptr) {
         return_error = EB_ErrorBadParameter;
     }
 
+    if (config->mbr_over_shoot_pct > 100) {
+        SVT_ERROR("Instance %u: The max bitrate overshoot percentage must be between [0, 100] \n",
+                  channel_number + 1);
+        return_error = EB_ErrorBadParameter;
+    }
+
     if (config->under_shoot_pct > 100) {
         SVT_ERROR("Instance %u: The undershoot percentage must be between [0, 100] \n",
                   channel_number + 1);
@@ -148,7 +154,26 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs_ptr) {
                   channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
+    if (config->gop_constraint_rc &&
+        ((config->rate_control_mode != SVT_AV1_RC_MODE_VBR) || config->intra_period_length < 119)) {
+        SVT_ERROR(
+            "Instance %u: Gop constraint rc is only supported with VBR mode when Gop size is "
+            "greater than 119 \n",
+            channel_number + 1);
+        return_error = EB_ErrorBadParameter;
+    }
 
+    if (config->gop_constraint_rc > 1) {
+        SVT_ERROR("Instance %u: Invalid gop_constraint_rc. gop_constraint_rc must be [0 - 1]\n",
+                  channel_number + 1);
+        return_error = EB_ErrorBadParameter;
+    }
+    if (config->gop_constraint_rc)
+        SVT_WARN(
+            "Instance %u: The GoP constraint RC mode is a work-in-progress project, and is only "
+            "available for demos, experimentation, and further development uses and should not be "
+            "used for benchmarking until fully implemented.\n",
+            channel_number + 1);
     if (config->force_key_frames &&
         (config->rate_control_mode != SVT_AV1_RC_MODE_CQP_OR_CRF ||
          config->pred_structure != SVT_AV1_PRED_RANDOM_ACCESS)) {
@@ -164,14 +189,6 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs_ptr) {
         return_error = EB_ErrorBadParameter;
     }
 
-    if (scs_ptr->max_input_luma_width % 8 &&
-        scs_ptr->static_config.compressed_ten_bit_format == 1) {
-        SVT_ERROR(
-            "Instance %u: Only multiple of 8 width is supported for compressed 10-bit inputs \n",
-            channel_number + 1);
-        return_error = EB_ErrorBadParameter;
-    }
-
     if (scs_ptr->max_input_luma_width > 16384) {
         SVT_ERROR("Instance %u: Source Width must be less than or equal to 16384\n",
                   channel_number + 1);
@@ -458,6 +475,11 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs_ptr) {
         return_error = EB_ErrorBadParameter;
     }
 
+    if (config->compressed_ten_bit_format) {
+        SVT_WARN("Instance %u: Compressed Ten Bit Format has been deprecated and has no effect\n",
+                 channel_number + 1);
+    }
+
     if (config->use_cpu_flags & EB_CPU_FLAGS_INVALID) {
         SVT_ERROR(
             "Instance %u: param '--asm' have invalid value.\n"
@@ -680,9 +702,8 @@ EbErrorType svt_av1_verify_settings(SequenceControlSet *scs_ptr) {
             channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
-
-    if (config->hierarchical_levels < 3 || config->hierarchical_levels > 5) {
-        SVT_ERROR("Instance %u: Only hierarchical levels 3-5 is currently supported.\n",
+    if (config->hierarchical_levels < 2 || config->hierarchical_levels > 5) {
+        SVT_ERROR("Instance %u: Only hierarchical levels 2-5 is currently supported.\n",
                   channel_number + 1);
         return_error = EB_ErrorBadParameter;
     }
@@ -976,6 +997,7 @@ EbErrorType svt_av1_set_default_params(EbSvtAv1EncConfiguration *config_ptr) {
     config_ptr->under_shoot_pct          = 25;
     config_ptr->over_shoot_pct           = 25;
     config_ptr->mbr_over_shoot_pct       = 50;
+    config_ptr->gop_constraint_rc        = 0;
     config_ptr->maximum_buffer_size_ms   = 1000; // default settings for CBR
     config_ptr->starting_buffer_level_ms = 600; // default settings for CBR
     config_ptr->optimal_buffer_level_ms  = 600; // default settings for CBR
@@ -1087,15 +1109,14 @@ void svt_av1_print_lib_params(SequenceControlSet *scs) {
             config->frame_rate_numerator,
             config->frame_rate_denominator);
         SVT_INFO(
-            "SVT [config]: bit-depth / color format / compressed 10-bit format\t\t: %d / "
-            "%s / %d\n",
+            "SVT [config]: bit-depth / color format \t\t\t\t\t: %d / "
+            "%s\n",
             config->encoder_bit_depth,
             config->encoder_color_format == EB_YUV400       ? "YUV400"
                 : config->encoder_color_format == EB_YUV420 ? "YUV420"
                 : config->encoder_color_format == EB_YUV422 ? "YUV422"
                 : config->encoder_color_format == EB_YUV444 ? "YUV444"
-                                                            : "Unknown color format",
-            config->compressed_ten_bit_format);
+                                                            : "Unknown color format");
 
         SVT_INFO("SVT [config]: preset / tune / pred struct \t\t\t\t\t: %d / %s / %s\n",
                  config->enc_mode,
@@ -1788,6 +1809,7 @@ EB_API EbErrorType svt_av1_enc_parse_parameter(EbSvtAv1EncConfiguration *config_
         {"maxsection-pct", &config_struct->vbr_max_section_pct},
         {"undershoot-pct", &config_struct->under_shoot_pct},
         {"overshoot-pct", &config_struct->over_shoot_pct},
+        {"mbr-overshoot-pct", &config_struct->mbr_over_shoot_pct},
         {"recode-loop", &config_struct->recode_loop},
         {"enable-stat-report", &config_struct->stat_report},
         {"scm", &config_struct->screen_content_mode},
@@ -1824,6 +1846,8 @@ EB_API EbErrorType svt_av1_enc_parse_parameter(EbSvtAv1EncConfiguration *config_
         {"resize-kf-denom", &config_struct->resize_kf_denom},
         {"qm-min", &config_struct->min_qm_level},
         {"qm-max", &config_struct->max_qm_level},
+        {"use-fixed-qindex-offsets", &config_struct->use_fixed_qindex_offsets},
+        {"gop-constraint-rc", &config_struct->gop_constraint_rc},
     };
     const size_t uint8_opts_size = sizeof(uint8_opts) / sizeof(uint8_opts[0]);
 
@@ -1909,7 +1933,6 @@ EB_API EbErrorType svt_av1_enc_parse_parameter(EbSvtAv1EncConfiguration *config_
         Bool       *out;
     } bool_opts[] = {
         {"use-q-file", &config_struct->use_qp_file},
-        {"use-fixed-qindex-offsets", &config_struct->use_fixed_qindex_offsets},
         {"enable-dlf", &config_struct->enable_dlf_flag},
         {"rmv", &config_struct->restricted_motion_vector},
         {"enable-tf", &config_struct->enable_tf},
